#include "metro/stationdata.h"

using namespace metro;

const id_station station::Data::invalid_id = (id_station)-1;

/// Container des stations et de leurs ID
station::IdContainer station::IdMgr::m_idStations;

/**
 * l'ID est unique pour un nom de station d'une ligne donnée, pour une étape donnée. 
 * C'est à dire que la station etoile de la ligne 1 n'aura pas le même ID que celle 
 * de la ligne 6. De plus, apres passage par une étape, la station aura une nouvelle 
 * ID. Ceci permet de générer un graph unique en tenant compte des etapes.
 * 
 * \param stationData information concernant une station
 * 
 * \return l'ID de la station
 */
id_station station::IdMgr::get_id(station::Data& stationData)
{
	static id_station idCnt = 0;

	if (stationData.id() != Data::invalid_id) {
		// l'ID existe déjà
		return stationData.id();
	}
	typedef IdContainer::index<key>::type station_set_by_key;
	const bmi::index<IdContainer, key>::type& c = bmi::get<key>(m_idStations);
	station_set_by_key::const_iterator itId = c.find(stationData.key());
	if (itId != c.end()) {
		// La station existe en mémoire
		stationData.m_id = itId->id();
	}
	else {
		// La station n'existe pas, on la crée avec un nouvel ID
		stationData.m_id = idCnt++;
		m_idStations.insert(stationData);
	}
	return stationData.id();
}

/**
 * \return le container indexé par l'ID des stations
 */
bmi::index<station::IdContainer, station::id>::type& station::IdMgr::get_by_id()
{
	return bmi::get<id>(m_idStations);
}

/**
 * \param stationData information de la station
 * 
 * \return l'ID de la station défini par 
 */
id_station station::get_id(Data& stationData)
{
	return IdMgr::get_id(stationData);
}

/**
 * \return le container indexé par l'ID des stations
 */
const station::container_by_id_t& station::get_by_id()
{
	return station::IdMgr::get_by_id();
}

station::Data::Data(const std::string& ligne, const std::string& nom, int nEtape)
: m_ligne(ligne)
, m_nom(nom)
, m_nEtape(nEtape)
, m_id(invalid_id)
{
}

std::string station::Data::key() const 
{
	std::ostringstream oss;
	oss << m_ligne << " - " << m_nom << " - " << m_nEtape;
	return oss.str();
}

id_station station::Data::id() const
{
	return m_id;
}

const std::string& station::Data::nom() const
{
	return m_nom;
}

const std::string& station::Data::ligne() const
{
	return m_ligne;
}

int station::Data::etape() const
{
	return m_nEtape;
}

bool station::Data::operator<(const Data& other)
{
	return m_id < other.m_id;
}
